///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CMEMPOOL_H
#define _CMEMPOOL_H
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _MEMPOOL_DEBUG_DISABLED	0
#define _MEMPOOL_DEBUG_GENERAL	1
#define _MEMPOOL_DEBUG_ADVANCED	2
#define _MEMPOOL_DEBUG_VERBOSE	3

/*
	_MEMPOOL_DEBUG_LEVEL = DISABLED
		Disables all emmory pool debugging. Fully optimized.

	_MEMPOOL_DEBUG_LEVEL = GENERAL
		Enables the memory pool to track memory allocations.

	_MEMPOOL_DEBUG_LEVEL = ADVANCED
		The memory pool will not only track what was allocated, but the number
			of bytes and the location in code (file & line) that reserved the memory.

	_MEMPOOL_DEBUG_LEVEL = VERBOSE
		Not only does the memory pool keep track of memory bytes and the location in code
			(file & line) that alloacted the them, but it also opens an independent console
			to which it writes verbose information. This option is very slow.
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#if _MEMPOOL_DEBUG_LEVEL >= 2
	#define Allocate(iSize, iCount) _dbg_Allocate(iSize, iCount, __FILE__, __LINE__)
	#define ReAllocate(lpMemory, iSize, iCount) _dbg_ReAllocate(lpMemory, iSize, iCount, __FILE__, __LINE__)
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CMemPool {
	typedef struct _TAG_MEM_ITEM {
		unsigned long iAddress;
		#if _MEMPOOL_DEBUG_LEVEL >= 2
			unsigned long iSize;
			char *sCodeFile;
			int iLineOfCode;
		#endif
		bool bFree;
	} MEMITEM, *LPMEMITEM;

	typedef struct _TAG_MEM_ITEMS {
		MEMITEM *Array;

		unsigned long iAllocated;
		unsigned long iAllocIncrement;
		unsigned long iUsedCount; //Number of used memory items.
	} MEMITEMS, *LPMEMITEMS;

	public:
		~CMemPool(void);
		CMemPool(void);

		#if _MEMPOOL_DEBUG_LEVEL >= 1
			CMemPool(unsigned int iAllocateIncrement);
		#endif

		#if _MEMPOOL_DEBUG_LEVEL >= 1
			bool FreeAll(void);
			int GetMemAllocationCount(void);
			int GetMemFreeCount(void);
			int GetSlotAllocationCount(void);

			bool Free(void *lpMemory);
			bool Destroy(void);
		#else
			void Free(void *lpMemory);
			void Destroy(void);
		#endif

		#if _MEMPOOL_DEBUG_LEVEL >= 1
			bool Initialize(unsigned int iAllocateIncrement);
		#endif
		bool Initialize(void);
		bool IsDebug(void);

		#if _MEMPOOL_DEBUG_LEVEL >= 2
			char sTempFile[MAX_PATH];
			unsigned int ReservedBytes(void);

			void *_dbg_Allocate(const unsigned long iSize, const unsigned long iCount,
				const char *sCodeFile, const int iLineOfCode);
			void *_dbg_ReAllocate(void *lpMemory, const unsigned long iSize, const unsigned long iCount,
				const char *sCodeFile, const int iLineOfCode);
		#else
			void *Allocate(const unsigned long iSize, const unsigned long iCount);
			void *ReAllocate(void *lpMemory, const unsigned long iSize, const unsigned long iCount);
		#endif

	private:
		void Lock(void);
		void Unlock(void);

		#if _MEMPOOL_DEBUG_LEVEL >= 1
			char sDebugText[2048];

			int FindFreeAllocSlot(void);
			int GetSlotByAddress(const unsigned long iAddress);
			int GetFileName(const char *sInPath, char *sOutFile);
			void DbgWrite(const char *sBuffer);
			void DbgWrite(const int iValue);

			#if _MEMPOOL_DEBUG_LEVEL >= 2
				unsigned int iReservedBytes;
			#endif

			#if _MEMPOOL_DEBUG_LEVEL >= 3
				bool OpenConsole(void);
				bool CloseConsole(void);
				HANDLE hStdOut;
			#endif

			MEMITEMS Items;
		#endif

		CRITICAL_SECTION csMemLock;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
